using System;
using System.Windows.Forms ;
using System.Drawing;
using System.ComponentModel ;
using System.Drawing.Design  ;
using System.Collections ;
using System.Diagnostics ;
using System.Runtime.InteropServices;
using System.Reflection ;
using ExtendedDataGrid.ColumnStyles.Events;
using ExtendedDataGrid.Controls ;


namespace ExtendedDataGrid.ColumnStyles
{
	[ComVisibleAttribute(false),DefaultEvent("SetCellStyle")]
	public class DataGridFormatTextBoxColumn: DataGridColumnStyleBase 
	{
		#region << Private Fields >> 
		private ImageList imgList=null;
		private ExtendedDataGrid.DataGridFormatTextBox edit=new ExtendedDataGrid.DataGridFormatTextBox();
		private int defaultImageIndex=-1;
		private string format="";
		private IFormatProvider formatInfo=null;
		private bool isInEditing =false;
		private ResizeType resizeType=ResizeType.Proportional ; 
		private event FormatTextBoxSetCellStyleEventHandler setStyle=null;
		private event EllipsisClickEventHandler ellipsis_Click=null;
		private StringFormat colFormat=null;
		private bool defaultAllowKeyboardInput=true;
		private bool defaultShowEllipsisButton=true ;

		private CurrencyManager listManager=null;

		private int currentRowNumber=-1;
		private object oldValue=DBNull.Value;
		private int xMargin=2;
		private int yMargin=1;
		private bool editingBegun=false;
		
		#endregion Private Fields 

		#region >> Constructors <<

		public DataGridFormatTextBoxColumn():this(null)
		{
		
		}

		public DataGridFormatTextBoxColumn(PropertyDescriptor prop):base(prop)
		{
			this.edit.Hide();
			this.edit.BorderStyle=BorderStyle.None ;
			this.edit.EllipsisClick+=new EventHandler(ellipsis_ClickHandler);
			this.colFormat=new StringFormat();
			this.edit.TextBox.KeyPress+=new KeyPressEventHandler(TextBox_KeyPress);
			
		}



		#endregion Constructors 

		#region <<  Properties  >>

		[DefaultValue(false)]
		public bool Multiline
		{
			get{return this.edit.Multiline ;}
			set{this.edit.Multiline=value;}
		}
		[DefaultValue(BorderStyle.None )]
		public BorderStyle BorderStyle
		{
			get{return this.edit.BorderStyle; }
			set{this.edit.BorderStyle=value;}
		}

		[DefaultValue(null)]
		public IFormatProvider FormatInfo
		{
			get{return this.formatInfo;}
			set{
				this.formatInfo=value;
				this.Invalidate();
			}
		}

		public bool DefaultAllowKeyboardInput
		{
			get{return this.defaultAllowKeyboardInput ;}
			set{
				this.defaultAllowKeyboardInput=value;
				this.ReadOnly=!value;
				this.edit.ReadOnly=!value;
				
				if (!value)
				{
					this.edit.Select(0,0);
				}
				else
					this.edit.SelectAll();

			}
		}
		

		[Browsable(false),EditorBrowsable(EditorBrowsableState.Never )]
		public override bool ReadOnly
		{
			get
			{
				return base.ReadOnly;
			}
			set
			{
				base.ReadOnly = value;
			}
		}


		/// <summary>
		/// This event is raised when the ellipsis(three dots)button is clicked 
		/// </summary>
		public event EllipsisClickEventHandler EllipsisClick
		{
			add{this.ellipsis_Click+=value;}
			remove{this.ellipsis_Click-=value;}
		}


		[DefaultValue(true),Browsable(true),Category("Behaviour")]
		public bool DefaultShowEllipsis
		{
			get{return this.defaultShowEllipsisButton ;}
			set{
				this.edit.ShowEllipsis=value;
				this.defaultShowEllipsisButton=value;
			}
		}

		
		[DefaultValue(EllipsisBackgroundStyle.ButtonStyle  )]
		public EllipsisBackgroundStyle EllipsisBackStyle
		{
			get{return this.edit.EllipsisBackStyle;}
			set{
				this.edit.EllipsisBackStyle =value;
				this.edit.Refresh();
			}
		}
		
		/// <summary>
		/// The textbox used for editing the cells inside this column
		/// </summary>
		[ReadOnly(true),Browsable(false)]
		public DataGridFormatTextBox TextBox
		{
			get{return this.edit;}
		}

		[DefaultValue(null),RefreshProperties(RefreshProperties.Repaint)]
		public ImageList ImageList
		{
			get
			{
				return this.imgList ;
			}
			set 
			{
				this.imgList=value ;
				this.edit.ImageList=value;
				if (value==null)
				{
					this.defaultImageIndex=-1;
				}

			}
		}


		/// <summary>
		/// The default index of the image to show in this column  
		/// </summary>
		[Description("The default index of the image to show in this column "),
		DefaultValue(-1),
		TypeConverter(typeof(ImageIndexConverter)),System.ComponentModel.EditorAttribute("System.Windows.Forms.Design.ImageIndexEditor",typeof(System.Drawing.Design.UITypeEditor))]
		public int DefaultImageIndex 
		{
			get
			{
				return this.defaultImageIndex ;
			}
			set
			{
				this.defaultImageIndex=value;
			}
		}



		[Category("Behaviour"),Description("Specifies how this column will autoresize when the grid will resize"),
		DefaultValue(ResizeType.Proportional )]
		public override ResizeType ResizeType
		{
			get{return this.resizeType ;}
			set{this.resizeType=value;}
		}
	

		/// <summary>
		/// This event is raised everytime the cell must be drawn
		/// </summary>
		[Description("This event is raised everytime the cell must be drawn")]
		public event FormatTextBoxSetCellStyleEventHandler SetCellStyle
		{
			add 
			{
					this.setStyle+=value ;
			}
			remove
			{
					this.setStyle-=value;
			}
		}


		public  string Format
		{
			get
			{
				return this.format ;
			}
			set
			{
			this.format=value;
			this.SetStringFormat();
			}
		}
		public override HorizontalAlignment Alignment
		{
			get
			{
				return base.Alignment;
			}
			set
			{
				base.Alignment = value;
				this.SetStringFormat();
			}
		}
		public override PropertyDescriptor PropertyDescriptor
		{
			get
			{
				return base.PropertyDescriptor;
			}
			set
			{
				base.PropertyDescriptor = value;
				if(this.colFormat!=null)
					this.SetStringFormat();
			}
		}



		#endregion Properties 

		#region <<  Overridings  >>

		protected override void Abort(int rowNum)
		{
			//DebugOut("Abort    rowNum="+rowNum.ToString());
			this.isInEditing=false;
			this.currentRowNumber=-1;
			this.edit.Hide();
			this.Invalidate();
		}
 

		protected override bool Commit(CurrencyManager dataSource, int rowNum)
		{
			//DebugOut("Commit   rowNum="+rowNum.ToString());
			object val=this.edit.Text ;

			bool success =true;

			if(this.isInEditing && !val.Equals(this.oldValue))
			{
				try
				{				
					if(rowNum!=dataSource.Position )
					{					  
						success=Util.SetValueInRow(dataSource.Current,this.MappingName,val);
						this.edit.Text=val.ToString();
					}
					else
					{
						if( !this.GetColumnValueAtRow(dataSource,rowNum ).Equals(val))
						{
							this.SetColumnValueAtRow(dataSource,rowNum,val);
						}
					}
				}
				catch(Exception ex)
				{
					success=false;
					throw new Exception("DataGridFormatTextBox.Commit Error :\n\n"+ex.Message,ex);
				}
			}

			this.currentRowNumber=-1;

			if (success)
				isInEditing=false ;
			this.edit.Hide();
			return success;
		}

		protected override void Dispose(bool disposing)
		{
			base.Dispose (disposing);
		}

		internal override void ReleaseFocus()
		{
			base.ReleaseFocus ();
			this.ConcedeFocus();
		}

//		protected override Size GetPreferredSize(Graphics g, object value)
//		{
//			Size sz;
//			sz = Size.Ceiling(g.MeasureString(this.GetFormatedValue(value), this.edit.Font ));
//			sz.Width = (sz.Width + ((this.xMargin * 2) + 1));
//			sz.Height = (sz.Height + this.yMargin);
//
//			if (this.imgList==null)
//				return sz;
//			else
//				return new Size(sz.Width+this.imgList.ImageSize.Width,(this.imgList.ImageSize.Height>sz.Height?this.imgList.ImageSize.Height:sz.Height+2));
//		}


//		protected override int GetMinimumHeight()
//		{
//			
//			return this.DataGridTableStyle.PreferredRowHeight ;
//		}
//
//
//		protected override int GetPreferredHeight(Graphics g, object value)
//		{
//
//			int baseHeight;
//			int curBlankPos=0;
//			int curBlankOccurs=0;
//
//			string formatedText=this.GetFormatedValue(value);
//			while ((curBlankPos != -1) && (curBlankPos < formatedText.Length))
//			{
//				curBlankPos = formatedText.IndexOf(" ", curBlankPos + 1);
//				curBlankOccurs = curBlankOccurs + 1;
//			}
//
//			baseHeight=base.FontHeight * curBlankOccurs + this.yMargin; 
// 
//			int supl = 4;
//			if (this.imgList==null)
//				return baseHeight+supl;
//			else
//				return (this.imgList.ImageSize.Height>baseHeight+supl?this.imgList.ImageSize.Height :baseHeight+supl);
//		
//		}
		


		protected override void SetDataGridInColumn(DataGrid value)
		{
			base.SetDataGridInColumn (value);
			if (this.edit.Parent!=null)
			{
				this.edit.Parent.Controls.Remove(this.edit);
			}

			if(value!=null)
			{
				value.Controls.Add(this.edit);
			}
			this.edit.TextBox.SetDataGrid(value);
		}



		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly)
		{
			this.Edit(source,rowNum,bounds,readOnly,"",true);
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText)
		{
			this.Edit (source, rowNum, bounds, readOnly, instantText,true);
		}

		protected override void Edit(CurrencyManager source, int rowNum, Rectangle bounds, bool readOnly, string instantText, bool cellIsVisible)
		{
			
			//DebugOut("Edit    instatntText="+(instantText==null?"(null)":instantText));
			base.Edit(source,rowNum,bounds,readOnly,instantText,cellIsVisible);
			this.listManager=source;
			this.currentRowNumber = rowNum;

			this.isInEditing=true;

			// take the caracteristics of the container grid
			System.Windows.Forms.DataGrid hostGrid=this.DataGridTableStyle.DataGrid ;

			//raise the event SetCellStyle
			FormatTextBoxSetCellStyleEventArgs e;
			if (rowNum % 2 ==0)
				e=new FormatTextBoxSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),hostGrid.BackColor,hostGrid.ForeColor,hostGrid.Font,this.defaultImageIndex,true,this.defaultAllowKeyboardInput,this.defaultShowEllipsisButton );
			else
				e=new FormatTextBoxSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),hostGrid.AlternatingBackColor ,hostGrid.ForeColor,hostGrid.Font,this.defaultImageIndex,true,this.defaultAllowKeyboardInput,this.defaultShowEllipsisButton );
			this.OnSetCellStyle(e);

			
			if (this.edit.Font!=e.CurrentFont)
				this.edit.Font=e.CurrentFont;

			this.edit.Text=this.GetFormatedValue(base.GetColumnValueAtRow(source, rowNum));


			if (cellIsVisible)
			{
				this.edit.Bounds=bounds;
				this.edit.BackColor= e.CurrentBackColor ;
				this.edit.ImageIndex=e.CurrentImageIndex ;
				this.edit.ForeColor=e.CurrentForeColor ;
				this.edit.ReadOnly=!e.CurrentAllowKeyboardInput  ;
				this.edit.ShowEllipsis=e.CurrentShowEllipsisButton;
				this.edit.TextAlign=this.Alignment ;
				this.edit.Show();
			}

			this.edit.Select();
			this.edit.Focus();

			if(e.CurrentAllowKeyboardInput )
			{
				this.edit.TextBox.Focus();
				this.edit.SelectAll();
				this.oldValue=this.edit.Text ;
			}
 
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum)
		{
			this.Paint(g,bounds,source,rowNum,null,null,false);
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, bool alignToRight)
		{
			this.Paint(g,bounds,source,rowNum,null,null,alignToRight);
		}

		protected override void Paint(Graphics g, Rectangle bounds, CurrencyManager source, int rowNum, Brush backBrush, Brush foreBrush, bool alignToRight)
		{

				//raise the event SetCellStyle
				FormatTextBoxSetCellStyleEventArgs e;
				Color backColor;Color foreColor;
				this.GetPaintColors(out backColor,out foreColor,rowNum,source.Position );

				e=new FormatTextBoxSetCellStyleEventArgs(Util.GetRowNumber(rowNum,source),backColor,foreColor,this.DataGridTableStyle.DataGrid.Font,this.defaultImageIndex, false ,this.defaultAllowKeyboardInput,this.defaultShowEllipsisButton );

				this.OnSetCellStyle(e);


			

				Rectangle imgRect=Rectangle.Empty;
				Rectangle txtBoxRect=Rectangle.Empty;

				this.BeginUpdate();


				// clear the background ;
				g.FillRectangle(new SolidBrush(e.CurrentBackColor),bounds);

				bounds.Inflate(-2,0);

				//

				if (e.CurrentImageIndex>-1)
					imgRect=this.GetImageRectangle(bounds);

				txtBoxRect=this.GetTextBoxBoxRectangle(bounds,imgRect,e.CurrentFont,false );

				// draw the Textbox content
				string txtValue="";


				if(!txtBoxRect.IsEmpty)
				{
					Brush brush=new SolidBrush(e.CurrentForeColor);
					object objValue=this.GetColumnValueAtRow(source,rowNum);
					if (objValue==DBNull.Value)
						txtValue=this.NullText ;
					else
						txtValue =this.GetFormatedValue(objValue);
					g.DrawString(txtValue,e.CurrentFont ,brush, txtBoxRect,this.colFormat);
					brush.Dispose();
				}

				//draw the image 
				if(!imgRect.IsEmpty)
				{
					Rectangle srcImgRect=new Rectangle(0,0,(this.imgList.ImageSize.Width>bounds.Width?bounds.Width:this.imgList.ImageSize.Width),(this.imgList.ImageSize.Height>bounds.Height?bounds.Height:this.imgList.ImageSize.Height )  );
					try
					{
						g.DrawImage(this.imgList.Images[e.CurrentImageIndex],imgRect,srcImgRect,GraphicsUnit.Pixel );
					}
					catch (ArgumentOutOfRangeException ex)
					{
						throw new ArgumentOutOfRangeException("ImageIndex",e.CurrentImageIndex,"ImageIndex inesistente nella ImageList\n"+ex.Message );
					}
					catch(Exception ex) 
					{
						throw new Exception(ex.Message );
					}
				}
				this.EndUpdate();
		}


		protected override void ReleaseHostedControl()
		{
			base.ReleaseHostedControl();

			if (this.edit.Parent!=null)
				this.edit.Parent.Controls.Remove(this.edit );
		}

		#endregion Overridings

		#region << Private methods >>


		private void GetPaintColors(out Color backColor, out Color foreColor ,  int rowNum , int sourcePosition)
		{
			DataGridTableStyle tableStyle=this.DataGridTableStyle ;
			if (rowNum==sourcePosition )
			{
				//foreColor=hostGrid.SelectionForeColor  ;
				if (this.isInEditing) 
				{
					backColor=tableStyle.BackColor  ;
					foreColor=tableStyle.ForeColor ;
				}
				else
				{
					backColor=tableStyle.SelectionBackColor ;
					foreColor=tableStyle.SelectionForeColor ;
				}
			}
			else
			{
				foreColor=tableStyle.ForeColor;
				if (rowNum % 2 ==0)
				{
					backColor=tableStyle.BackColor;
				}
				else 
				{
					backColor=tableStyle.AlternatingBackColor;
				}
			}

			//Debug.Assert(foreColor!=Color.White );
		}

	
		
		
		void SetStringFormat()
		{
			if (this.Alignment==HorizontalAlignment.Left) 
				this.colFormat.Alignment=StringAlignment.Near ;
			else if (this.Alignment==HorizontalAlignment.Right )
				this.colFormat.Alignment=StringAlignment.Far ;
			else
				this.colFormat.Alignment=StringAlignment.Center ;
		}


		//internal use
		string GetFormatedValue(object value) 
			{
			string result;
			if ((value as DBNull) != null)
					result= base.NullText; 
			else
				if ((this.Format  != null) && ((this.Format.Length != 0) && ((value as IFormattable) != null)))
				{
					try
					{
						result = ((IFormattable) value).ToString(this.format, this.formatInfo);
	 
					}
					catch (Exception ex)
					{
						result=ex.Message;result="";
					}
				}
				else
					result=value.ToString();
 
			return result; 
			}

		private Rectangle GetImageRectangle(Rectangle cellBounds)
		{
			if (this.imgList==null)
				return Rectangle.Empty;
			//if the height of the cell is greater than the image's one I have to 
			//center the image on vertical 
			Rectangle rctResult=Rectangle.Empty;
			if (this.imgList.ImageSize.Height<cellBounds.Height )
			{
				int top =0;
				top=(cellBounds.Height-this.imgList.ImageSize.Height )/2;
				rctResult=new Rectangle(cellBounds.X,cellBounds.Y+top,this.imgList.ImageSize.Width,this.imgList.ImageSize.Height  );
			}
			else
				rctResult=new Rectangle(cellBounds.X,cellBounds.Y,this.imgList.ImageSize.Width,cellBounds.Height);
			return rctResult;
		}


		private Rectangle GetTextBoxBoxRectangle(Rectangle cellBounds,Rectangle rctImage, Font font,bool calledFromEdit )
		{
			if (calledFromEdit)
			{
				cellBounds.Offset(this.xMargin, (2 * this.yMargin));
				cellBounds.Width -= 2*this.xMargin;
				cellBounds.Height-= 4 * this.yMargin; 
			}
			else
			{
				cellBounds.Offset(0, 2 * this.yMargin);
				cellBounds.Height -= 4 * this.yMargin;
			}

			// push the textbox to the right to make place for the image 

			int imgSpace=0;
			int ellipsisSpace=0;
			if(this.Alignment!=HorizontalAlignment.Left && this.edit.ShowEllipsis )
				ellipsisSpace=this.edit.EllipsisWidth ;

			if(!rctImage.IsEmpty)
			{
				imgSpace=rctImage.Width ; // +2*this.offset;
			}

			Rectangle rctTxtBox=Rectangle.Empty;
			int top=0;
			if(imgSpace <= cellBounds.Width  )
			{
				if (this.edit.Multiline)
				{
					rctTxtBox=new Rectangle(cellBounds.X+imgSpace,
						cellBounds.Y,
						cellBounds.Width-imgSpace-ellipsisSpace,
						cellBounds.Height );
					
				}
				else
				{
					top=(cellBounds.Height>font.Height?(cellBounds.Height-font.Height)/2:0);

					rctTxtBox=new Rectangle(cellBounds.X+imgSpace ,
						cellBounds.Y+top,
						cellBounds.Width-imgSpace-ellipsisSpace,
						font.Height );
				}
			}

			return rctTxtBox;
		}


		#endregion Private methods 

		#region >>  ProtectedMethods  <<
		protected virtual void OnEllipsisClick(EllipsisClickEventArgs e)
		{
			if(this.ellipsis_Click!=null)
			{
				this.ellipsis_Click(this,e);
			}
		}


		
		protected virtual void OnSetCellStyle(Events.FormatTextBoxSetCellStyleEventArgs e)
		{
			if(this.setStyle!=null)
			{
					this.setStyle(this,e);
			}
		}
		protected override void ConcedeFocus()
		{
			this.edit.Hide();
			this.isInEditing=false;
			this.editingBegun=false;
			base.ConcedeFocus ();
		}

		#endregion ProtectedMethods



		private void ellipsis_ClickHandler(object sender, EventArgs e)
		{
			int rowIndex ;
			if(this.currentRowNumber<this.listManager.Count && this.currentRowNumber>-1 )
				rowIndex=this.currentRowNumber;
			else
				rowIndex=this.listManager.Count;

			object oldFieldVal ;

			EllipsisClickEventArgs eArg=new EllipsisClickEventArgs(rowIndex,this.edit.Text,this.listManager.Current );
			oldFieldVal=Util.GetValueFromRow(eArg.CurrentRow ,this.MappingName);
			OnEllipsisClick(eArg);
			object newVal=Util.GetValueFromRow(eArg.CurrentRow,this.MappingName);
			if (!oldFieldVal.Equals(Util.GetValueFromRow(eArg.CurrentRow,this.MappingName)))
			{
				if(this.editingBegun==false )
				{
					this.edit.Text=this.GetFormatedValue(newVal);
					this.ColumnStartedEditing(this.edit);
					this.editingBegun=true;
				}
			}
		}



		private void TextBox_KeyPress(object sender, KeyPressEventArgs e)
		{

			if(this.edit.ReadOnly==false && this.editingBegun==false  && Util.IsPrintable(e.KeyChar  ))
			{
				this.ColumnStartedEditing(this.edit);
				this.editingBegun=true;
			}
		}


		private void DebugOut(string text)
		{
		Debug.WriteLine (text+"      Class:DataGridFormatTextBox");
		}



	}//end class DataGridFormatTextBoxColumn

	namespace Events
	{

		public delegate void EllipsisClickEventHandler(object sender,EllipsisClickEventArgs e);

		public class EllipsisClickEventArgs:EventArgs
		{
			private int rowNum ;
			private object currentRow;
			private string textValue;
			

			public EllipsisClickEventArgs(int rowNumber,string txtVal,object curRow)
			{
				this.rowNum=rowNumber;
				this.textValue=txtVal;
				this.currentRow=curRow;
			}

			/// <summary>
			/// Gets the index of the row for which this event has been raised. 
			/// On a new row this index is equal to DataTable.Count
			/// </summary>
			public int RowNumber
			{
				get{return this.rowNum;}
				set{this.rowNum=value;}
			}

			/// <summary>
			/// Gets the text the user has input in the textbox of the cell; 
			/// The CurrentRow may not contain this text at this time 
			/// </summary>
			[ReadOnly(true)]
			public string TextValue
			{
				get{return this.textValue;}
			}

			/// <summary>
			/// You can change the values in the CurrentRow
			/// </summary>
			[ReadOnly(true)]
			public object CurrentRow
			{
				get{return this.currentRow ;}
			}

		
		}
		

		public delegate void FormatTextBoxSetCellStyleEventHandler(object sender,FormatTextBoxSetCellStyleEventArgs e);

		/// <summary>
		/// Class SetCellStyleEventArgs  
		/// This type is the argument of the  SetCellStyle  event
		/// </summary>
		public class FormatTextBoxSetCellStyleEventArgs:EventArgs
		{
			#region Private Fields 
			private Color currentBackColor;
			private Color currentForeColor;
			private int currentImageIndex=-1;
			private Font currentFont=null;
			private bool isInEditing=false;
			private int currentRowNumber=-1;
			private bool currentAllowKeyboardInput;
			private bool currentShowEllipsisButton;
			#endregion Private Fields 

			#region Constructors
			public FormatTextBoxSetCellStyleEventArgs(int curRowIndex,Color curBackColor,Color curForeColor,Font curFont,int curImageIndex, bool inEditing ,bool curAllowKeyboardInput, bool curShowEllipsisButton )
			{
				this.currentRowNumber=curRowIndex ;
				this.currentBackColor=curBackColor; 
				this.currentForeColor=curForeColor;
				this.currentFont=curFont;
				this.currentImageIndex=curImageIndex;
				this.isInEditing=inEditing ;
				this.currentShowEllipsisButton=curShowEllipsisButton;
				this.currentAllowKeyboardInput=curAllowKeyboardInput;
			}
			#endregion constructors 

			#region Properties

			/// <summary>
			/// Gets/Sets the BackColor of the cell for which this event has been raised  
			/// </summary>
			public Color CurrentBackColor 
			{
				get
				{
					return this.currentBackColor ;
				}
				set
				{
					this.currentBackColor=value;
				}
			}
			
			/// <summary>
			/// Gets/Sets a boolean value indicating whether the EllipsisButton should be visible
			/// </summary>
			public bool CurrentShowEllipsisButton
			{
				get{return this.currentShowEllipsisButton ;}
				set{this.currentShowEllipsisButton=value;}
			}


			/// <summary>
			/// Gets/Sets a boolean value indicating whether the control allows modificatins with the keyboard 
			/// </summary>
			public bool CurrentAllowKeyboardInput
			{
				get
				{
				return this.currentAllowKeyboardInput;
				}
				set
				{
				this.currentAllowKeyboardInput=value;
				}
			}

//			/// <summary>
//			/// Gets/Sets the ReadOnly property of the cell for which this event has been raised 
//			/// </summary>
//			public bool CurrentReadOnly
//			{
//				get{return this.currentReadOnly;}
//				set{this.currentReadOnly=value;}
//			}
//
			/// 
			/// Gets/Sets the ForeColor of the cell for which this event has been raised  
			/// 
			public Color CurrentForeColor 
			{
				get
				{
					return this.currentForeColor ;
				}
				set
				{
					this.currentForeColor=value;
				}
			}

			/// <summary>
			/// Gets/sets the index of the image to show in the cell for which this event has been raised 
			/// </summary>
			public int CurrentImageIndex 
			{
				get 
				{
					return this.currentImageIndex; 
				}
				set 
				{
					this.currentImageIndex=value;
				}
			}

			/// <summary>
			/// Gets/Sets the font for  the cell for which this event has been raised 
			/// </summary>
			public Font CurrentFont
			{
				get
				{
					return this.currentFont ;
				}
				set
				{
					if (value==null)
						throw new Exception("Invalid Font");
					else
						this.currentFont=value;
				}
			}


			/// <summary>
			/// Indicates whether the cell(for which this event has been raised) is being edited 
			/// </summary>
			public bool IsInEditing
			{
				get
				{
					return this.isInEditing ;
				}
			}


			/// <summary>
			/// Gets the index of the row for which this event has been raised 
			/// </summary>
			public int CurrentRowNumber
			{
				get
				{
					return this.currentRowNumber ;
				}
			}


			#endregion Properties 
		}
	
	}//namespace Events
}

