using System;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using System.ComponentModel;
using System.Collections;
using System.Globalization;
using System.Drawing ;
using System.Diagnostics ;
using ExtendedDataGrid.Controls.Events ;

namespace ExtendedDataGrid.Controls
{
	internal class DateEdit : TextBox
	{
		private string editMask;		// display format (editMask with input chars replaced by input char)
		private char maskInputChar = '_';
		private string m_regex = @"[0-9]";
		private string editFormat="d";
		private string initEditFormat;
		private string displayFormat;
		private System.Globalization.DateTimeFormatInfo  formatInfo=null;
		private bool hasFocus=false;
		private object currentValue=DBNull.Value ;
		private string currentText="";
		private string nullDisplayText="(null)";
		private int caret;
		private Hashtable hash;
		private event DateCancelEventHandler validating;

		#region <<  Constructors  >>
		public DateEdit()
		{
			this.hash=new Hashtable();
			this.formatInfo=CultureInfo.CurrentCulture.DateTimeFormat;
			this.displayFormat= CultureInfo.CurrentCulture.DateTimeFormat.LongDatePattern;
			BuildMask();
			this.initEditFormat=this.editFormat ;
			BuildPosNdx();
			base.Text = "";
			this.currentText=this.editMask ;
		
			
			// disable context menu since it bypasses Ctrl+V handler
			this.ContextMenu = null;
		}

		#endregion 



		[Description("Sets the input char"), Category("Behaviour"), 
		DefaultValue('_'),
		RefreshProperties(RefreshProperties.All)]
		public char InputChar
		{
			get{return this.maskInputChar;}
			set
			{
				this.maskInputChar = value;
				BuildMask();
				base.Text=this.editMask ;
			}
		}
		
		[ReadOnly(true),Category("Behaviour")]
		public string EditMask
		{
			get{return this.editMask ;}
		}

		[DefaultValue("(null)"),Browsable(true),Localizable(true)]
		public string NullDisplayText
		{
			get{return this.nullDisplayText;}
			set{this.nullDisplayText=value;}
		}
		


		/// <summary>
		/// The format used when the date is being edited
		/// This format is limited to strings containing dd, MM , yy or yyyy and separators 
		/// Samples of valid formats : "MM/dd/yy", "MM/dd/yyyy" , "dd/MM/yy" , "dd/MM/yyyy"
		/// </summary>
		[Browsable(true),Category("Behaviour"),
		RefreshProperties(RefreshProperties.All)]
		public string EditFormat
		{
			get{return this.editFormat ;}
			set{
				if(this.VerifyEditFormat(value))
				{
					this.editFormat=value;
					this.BuildMask();
					base.Text=this.editMask ;
				}
				else
					throw new FormatException("The only valid formats are: d or strings containing MM, dd, yy, yyy and separators");
			}
		}

		public bool ShouldSerializeEditFormat()
		{
			return this.editFormat != this.initEditFormat ;
		}
		public void ResetEditFormat()
		{
		this.editFormat="";
		this.BuildMask();
		}


		/// <summary>
		/// The format used to display the date. Any .NET DateTimeFormat is valid 
		/// </summary>
		[Browsable(true),Category("Behaviour")]
		public string DisplayFormat
		{
			get{return this.displayFormat;}
			set{this.displayFormat=value;}
		}
		public bool ShouldSerializeDisplayFormat()
		{return this.displayFormat!= CultureInfo.CurrentCulture.DateTimeFormat.LongDatePattern;}
		

		/// <summary>
		/// The default value of this property is CultureInfo.CurrentCulture.DateTimeFormat
		/// You can use this property to set the date format for other culture 
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public System.Globalization.DateTimeFormatInfo  DateTimeFormatInfo
		{
			get{return this.formatInfo ;}
			set{this.formatInfo=value;this.BuildMask();base.Text=this.editMask ;}
		}

		public void ResetDateTimeFormatInfo()
		{
			this.formatInfo=CultureInfo.CurrentCulture.DateTimeFormat ;
			this.BuildMask();base.Text=this.editMask ;
		}


		public new event DateCancelEventHandler Validating
		{
			add{this.validating+=value;}
			remove
			{
				try
				{
				this.validating-=value;
				}
				catch(Exception ex)
				{
				throw new ArgumentException(ex.Message ,"Validating");
				}
			}
		}


		/// <summary>
		/// Returns the formated Text value of this control 
		/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),ReadOnly(true),
		Description("Returns the formatted Text value of this control")]
		public  string FormattedText
		{
			get{
				if(base.Text==this.editMask || base.Text==this.nullDisplayText || !this.IsDateValid  )
					return this.nullDisplayText;
				else
					return base.Text ; 
			}
		}

		[ReadOnly(true),Browsable(true),Description("Gets the Text value of this control as is without formatting it"),DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public new string Text
		{get{return base.Text;}}


		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),Browsable(false),ReadOnly(true)]
		public override bool Multiline
		{
			get{return base.Multiline;}
		}


		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),Browsable(false),ReadOnly(true)]
		public new int SelectionStart
		{
			get{return base.SelectionStart;}
		}


		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),Browsable(false),ReadOnly(true)]
		public override int SelectionLength
		{
			get{return base.SelectionLength;}
		}


		[ReadOnly(true),Browsable(false)]
		public override int MaxLength
		{
			get{return base.MaxLength;}
		}
		
		protected override void WndProc(ref Message m)
		{
			if(m.Msg==0x101) //WM_KEYUP
				return ;
			base.WndProc (ref m);
		}


		protected override void OnValidating(CancelEventArgs e)
		{
			if(this.validating!=null)
			{
				base.OnValidating (e);
				DateCancelEventArgs eArg=new DateCancelEventArgs(this.IsDateValid);
				this.validating(this,eArg);
				e.Cancel=eArg.Cancel ;
			}
		}


		internal void SetText(string newVal)
		{
		base.Text=newVal;
		}

		protected override void OnGotFocus(EventArgs e)
		{

			base.OnGotFocus (e);
			if(this.hasFocus)
				return ;
			this.hasFocus=true ;
			if(this.currentValue.Equals(DBNull.Value))
				base.Text=this.editMask;
			else
			{
				DateTime dt=(DateTime)this.currentValue;
				base.Text=dt.ToString(this.editFormat,this.formatInfo);
			}
			this.currentText=base.Text;
			
		}

		protected override void OnLostFocus(EventArgs e)
		{
			
			
			object dt;
			if(base.Text!=this.currentText)
			{
				try
				{
					dt=this.Value ;
					this.currentValue=dt;
					if(!dt.Equals(DBNull.Value ))
						base.Text=((DateTime)dt).ToString(this.displayFormat,this.formatInfo);
					else
						base.Text=this.nullDisplayText;
				}
				catch
				{
					//this.currentValue=
				}
			}
			else
			{
				try
				{
					if(!this.currentValue.Equals(DBNull.Value ))
						base.Text=((DateTime)this.currentValue).ToString(this.displayFormat,this.formatInfo);
					else
						base.Text=this.nullDisplayText;
				}
				catch
				{}
			}
			this.hasFocus=false;
			base.OnLostFocus (e);
		}


		protected override bool ProcessCmdKey(ref Message msg, Keys keyData)
		{
			// NOTES: 
			//	1) break; causes warnings below
			//	2) this.caret tracks caret location, always the start of selected char
			//	3) code below is based on MaskedEdit, since our format is fixed
			//		there may be optimizations possible
			int start = base.SelectionStart;
			int len = base.SelectionLength;
			int end = start + base.SelectionLength - 1;
			string actualText = base.Text;
			int prev;
			
			// handle startup, runs once
			if(editMask[start] != this.maskInputChar)
			{
				start = Next(-1);
				len = 1;
			}

			switch(keyData)
			{
				case Keys.Left:
					prev = Prev(start);
					if(prev != start)
					{
						base.SelectionStart = prev;
						base.SelectionLength = 1;
					}
					this.caret = prev;
					return true;
				case Keys.Left | Keys.Shift:
					if((start < this.caret) || (start == this.caret && len <= 1))
					{
						// enlarge left
						prev = Prev(start);
						base.SelectionStart -= (start - prev);
						base.SelectionLength = len + (start - prev);
					}
					else
					{
						// shrink right
						base.SelectionLength = len - (end - Prev(end));
					}
					return true;
				case Keys.Right:
					prev = Next(start);
					if(prev != start)
					{
						base.SelectionStart = prev;
						base.SelectionLength = 1;
					}
					this.caret = prev;
					return true;
				case Keys.Right | Keys.Shift:
					if(start < this.caret)
					{
						// shrink left
						prev = Next(start);
						base.SelectionStart += (prev - start);
						base.SelectionLength = len - (prev - start);
					}
					else if(start == this.caret)
					{
						// enlarge right
						prev = Next(end);
						base.SelectionLength = len + (prev - end);
					}
					return true;
				case Keys.Delete | Keys.Back  :
					// delete selection, replace with input format
					base.Text = actualText.Substring(0, start) + editMask.Substring(start, len) + actualText.Substring(start + len);
					base.SelectionStart = start;
					base.SelectionLength = 1;
					this.caret = start;
					return true;
				case Keys.Home:
				case Keys.Left | Keys.Control:
				case Keys.Home | Keys.Control:
					base.SelectionStart = Next(-1);
					base.SelectionLength = 1;
					this.caret = base.SelectionStart;
					return true;
				case Keys.Home | Keys.Shift:
					if(start <= this.caret && len <= 1)
					{
						// enlarge left
						prev = Next(-1);
						base.SelectionStart -= (start - prev);
						base.SelectionLength = len + (start - prev);
					}
					else
					{
						// shrink right
						prev = Next(-1);
						base.SelectionStart = prev;
						base.SelectionLength = (this.caret - prev) + 1;
					}
					return true;
				case Keys.End:
				case Keys.Right | Keys.Control:
				case Keys.End | Keys.Control:
					base.SelectionStart = Prev(base.MaxLength);
					base.SelectionLength = 1;
					this.caret = base.SelectionStart;
					return true;
				case Keys.End | Keys.Shift:
					if(start < this.caret)
					{
						// shrink left
						prev = Prev(base.MaxLength);
						base.SelectionStart = this.caret;
						base.SelectionLength = (prev - this.caret + 1);
					}
					else if(start == this.caret)
					{
						// enlarge right
						prev = Prev(base.MaxLength);
						base.SelectionLength = len + (prev - end);
					}
					return true;
				case Keys.V | Keys.Control:
				case Keys.Insert | Keys.Shift:
					// paste from clipboard
					IDataObject iData = Clipboard.GetDataObject();

					// assemble new text
					string newText = actualText.Substring(0, start)
						+ (string)iData.GetData(DataFormats.Text)
						+ actualText.Substring(start + len);

					// check if data to be pasted is convertable to inputType
					DateTime dt;;
					try
					{
						dt = DateTime.Parse(newText);
						base.Text = dt.ToString(this.editFormat );

						// reset selection
						base.SelectionStart = start;
						base.SelectionLength = len;
					}
					catch
					{
						// do nothing
					}
					
					return true;
				default:
					return base.ProcessCmdKey(ref msg, keyData);
			}
		}

		protected override void OnMouseUp(MouseEventArgs e)
		{
			// reset selection to include input chars
			int start = base.SelectionStart;
			int orig = start;
			int len = base.SelectionLength;

			// reset selection start
			if(start == base.MaxLength || editMask[start] != this.maskInputChar)
			{
				// reset start
				if(Next(start) == start)
					start = Prev(start);
				else
					start = Next(start);

				base.SelectionStart = start;
			}

			// reset selection length
			if(len < 1)
				base.SelectionLength = 1;
			else if(editMask[orig + len - 1] != this.maskInputChar)
			{
				len += Next(start + len) - (start + len);
				base.SelectionLength = len;
			}

			this.caret = start;
			base.OnMouseUp(e);
		}


		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Browsable(false)]
		public bool IsDateValid
		{
			get
			{
				if(base.Text == this.editMask)
					return false;
				if(base.Text.IndexOf(this.InputChar)>-1)
					return false;

				DateTime val=DateTime.Now ;
				bool result=true ;

				try
				{
					val = DateTime.Parse(base.Text);
				}
				catch
				{
					result= false;
				}

				return result;
			}
		}

/// <summary>
/// This property can be bound to a table field 
/// </summary>
		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden), Browsable(true),Bindable(true)]
		public object Value
		{
			get
			{
				if(!this.hasFocus)
					return this.currentValue ;
				else
					if(base.Text==this.editMask)
						return DBNull.Value ;

				bool err=false;
				DateTime ret=DateTime.Now;
				string inputText=base.Text.Replace(this.maskInputChar.ToString(),"");
				if (inputText.Length<this.editMask.Length-2)
					err=true;
				else
				{
					
					try
					{		
						ret = DateTime.Parse(inputText,this.formatInfo,DateTimeStyles.None);
						return ret;
					}
					catch
					{
						err=true ;
					}
				}
				if(err)
					return DBNull.Value ;
				else
					return ret;
			}
			set
			{
				if(value.Equals(DBNull.Value) )
				{
					if(this.hasFocus)
						base.Text = editMask;
					else
						base.Text="";
					this.currentValue=DBNull.Value ;
				}
				else
				{
					DateTime dt=DateTime.Now;
					bool err=false;
					string msg="";
					this.currentValue=value ;
					try
					{
						dt=(DateTime)value;
					}
					catch(Exception ex)
					{
						err=true ;
						msg=ex.Message ;
					}
					if(err)
					{
						this.currentValue=DBNull.Value ;
						if(this.hasFocus)
							base.Text=editMask;
						else
							base.Text="";
					}
					else
						if(this.hasFocus)
						base.Text=dt.ToString(this.editFormat );
					else
						base.Text=dt.ToString(this.displayFormat);
				}
			}
		}
		

		/// <summary>
		/// You should check the IsDateValid before calling this method.
		/// If the text(the user has input) is cannot be converted to a valid DateTime value,
		/// a FormatException will be raised   
		/// </summary>
		/// <returns>Return a DateTime representation of the edited text</returns>
		public DateTime GetDateTimeValue()
		{
			DateTime result=DateTime.Now ;
			try
			{
			result=DateTime.Parse(base.Text,this.formatInfo,DateTimeStyles.None);
			}
			catch(FormatException ex)
			{
				throw(new FormatException(ex.Message ));
			}
			return result;
		}


		protected override void OnKeyPress(KeyPressEventArgs e)
		{
			base.OnKeyPress(e);
			int start = base.SelectionStart;
			int len = base.SelectionLength;
			int prev;

			// Handle Backspace -> replace previous char with inpchar and select
			if(e.KeyChar == 0x08)
			{
				string actualText = base.Text;
				prev = Prev(start);
				if(prev != start)
				{
					base.Text = actualText.Substring(0, prev) + this.maskInputChar.ToString() + actualText.Substring(prev + 1);
					base.SelectionStart = prev;
					base.SelectionLength = 1;
					
				}
				this.caret = prev;
				e.Handled = true;
				return;
			}
			
			// update display if valid char entered
			if(IsDateValidChar(e.KeyChar, (int)this.hash[start]))
			{
				// assemble new text
				string newText = base.Text.Substring(0, start)+e.KeyChar.ToString();

				if(start + len != base.MaxLength && len>0)
				{
					newText += editMask.Substring(start + 1, len - 1);
					newText += base.Text.Substring(start + len);
				}
				else
					newText += editMask.Substring(start + 1);

				base.Text = newText;

				// select next input char
				start = Next(start);
				base.SelectionStart = start;
				this.caret = start;
				base.SelectionLength = 1;
			}
			e.Handled = true;
		}


		private bool IsDateValidChar(char input, int pos)
		{
			// validate input char against editMask
			return Regex.IsMatch(input.ToString(), m_regex);
		}


		private int Prev(int startPos)
		{
			// return previous input char position
			// returns current position if no input chars to the left
			int start = startPos;
			int ret = start;

			while(start > 0)
			{
				start--;
				if(editMask[start] == this.maskInputChar)
					return start;
			}
			return ret;			
		}


		private int Next(int startPos)
		{
			// return next input char position
			// returns current position if no input chars to the left
			int start = startPos;
			int ret = start;
			
			while(start < base.MaxLength - 1)
			{
				start++;
				if(this.editMask[start] == this.maskInputChar)
					return start;
			}

			return ret;			
		}


		private void BuildMask()
		{
			if (this.editFormat.Length==0 || this.editFormat=="d")
				this.editFormat=this.formatInfo.ShortDatePattern ;
			if (this.displayFormat.Length==0)
				this.displayFormat=this.formatInfo.LongDatePattern ;
			string separator = this.formatInfo.DateSeparator ;
			int posD,posM,posY ;
			string dblInput=this.maskInputChar.ToString()+this.maskInputChar.ToString();
			string formatYear =(this.editFormat.IndexOf("yyyy")>-1?"yyyy":"yy");
			string inputYear=formatYear.Replace('y',this.InputChar);
			posD=this.editFormat.IndexOf("d");
			posM=this.editFormat.IndexOf("M");
			posY=this.editFormat.IndexOf("y");

			
			this.editMask=dblInput+separator+dblInput+separator+inputYear;
			this.editFormat=@"MM/dd/"+formatYear;

			if (posD<posM && posM<posY )
			{
				this.editMask=dblInput+separator+dblInput+separator+inputYear;
				this.editFormat=@"dd/MM/"+formatYear;
			}
			if (posY<posM && posM<posD )
			{
				this.editMask=inputYear+separator+dblInput+separator+dblInput;
				this.editFormat=formatYear+@"/dd/MM";
			}

			if(posM<posY && posY<posD)
			{
				this.editMask=dblInput+separator+inputYear+separator+dblInput;
				this.editFormat=@"MM/"+formatYear+@"/dd";
			}
			if(posD<posY && posY<posM)
			{
				this.editMask=dblInput+separator+inputYear+separator+dblInput;
				this.editFormat=@"dd/"+formatYear+@"/MM";
			}

			if( posY<posD && posD<posM )
			{
				this.editMask=inputYear+separator+dblInput+separator+dblInput;
				this.editFormat=formatYear+@"/dd/MM";
			}

			this.editMask=this.editFormat.Replace("/",separator);
			this.editMask=this.editMask.Replace('d',this.maskInputChar);
			this.editMask=this.editMask.Replace('M',this.maskInputChar);
			this.editMask=this.editMask.Replace('y',this.maskInputChar);
			base.MaxLength = editMask.Length;

		}


		private bool VerifyEditFormat(string format)
		{
		if(format=="d")
			return true;
		if(format.Length>10)
			return false;
		format=format.Replace("/","");
		format=format.Replace("d","");
		format=format.Replace("M","");
		format=format.Replace("y","");
		return format.Length==0;
		}


		private void BuildPosNdx()
		{
			this.hash.Clear();
			for(int i=0 ;i< this.editMask.Length;i++ )
				if(this.editMask[i]==this.maskInputChar)
					this.hash.Add(i,i);

		}
	}
	namespace Events
	{
		#region << DateCancelEvent - event>>
		public delegate void DateCancelEventHandler(object sender ,DateCancelEventArgs e );
		public class DateCancelEventArgs:System.ComponentModel.CancelEventArgs 
		{
			private bool isDateValid=false; 
			public DateCancelEventArgs(bool isValid)
			{
				this.isDateValid=isValid;
			}

			[ReadOnly(true)]
			public bool IsDateValid
			{
				get{return this.isDateValid;}
			}
		}
		#endregion
	}// namespace Events
}
